#include <array>
#include <cassert>
#include <fstream>
#include <algorithm>
#include "SFML/Window.hpp"
#include "SFML/Graphics.hpp"
#include "constexpr.h"
#include "prefPathWithAPI.h"
#include "scoreGraphics.h"

ScoresGraphics::ScoresGraphics(std::ofstream& errorLog, const PrefPathFinder& prefPath, const sf::Font& arial):
	scoresData{
		SkillLevelScore{arial, "Easy", EASY_GAME},
		SkillLevelScore{arial, "Intermediate", INTERMEDIATE_GAME},
		SkillLevelScore{arial, "Hard", HARD_GAME}
	}
{
	title.setFont(arial);
	title.setString("Best scores");
	title.setCharacterSize(24);
	title.setFillColor(sf::Color::White);
	title.setPosition(SCREENW/2 - title.getLocalBounds().width / 2, SQR_SIZE);
	
	loadScoreFile(errorLog, prefPath.getPath() + EASY_FILE_NAME, EASY_GAME);
	loadScoreFile(errorLog, prefPath.getPath() + INTERM_FILE_NAME, INTERMEDIATE_GAME);
	loadScoreFile(errorLog, prefPath.getPath() + HARD_FILE_NAME, HARD_GAME);
	
	for( auto &data : scoresData )
	{
		sortScoresArray(data.scores);
	}
}

void ScoresGraphics::drawScores(sf::RenderWindow& renderer) const
{
	renderer.draw(title);
	for( const auto &skillScore : scoresData )
	{
		renderer.draw(skillScore.title);
		for( unsigned i{0} ; i < SCORE_NUM_PER_SKILL - 1 ; ++i )
		{
			renderer.draw(skillScore.scores[i].text);
		}
	}
}

void ScoresGraphics::loadScoreFile(std::ofstream& errorLog, const std::string& scoreFilePath, unsigned skill)
{
	assert( skill < GAME_MAX );
	if( std::ifstream scoreFile{scoreFilePath} )
	{
		for(unsigned i{0} ; i < GAME_MAX - 1 ; ++i )
		{
			if( !( scoreFile >> scoresData[skill].scores[i].score ) )
			{
				errorLog << "Error: couldn't read all the scores data.\n";
			}
		}
	}
	else{
		errorLog << "Error: couldn't open file '" << scoreFilePath << "' to load score of skill number #" << skill << " .\n"; 
	}
}

void ScoresGraphics::createScoreTexts()
{
	for( unsigned i{0} ; i < scoresData.size() ; ++i )
	{
		scoresData[i].setScoresTextsPosition(i);
	}
}

void ScoresGraphics::addCurrentScoreToBest(long unsigned newScore, unsigned skill)
{
	assert( skill < GAME_MAX );
	scoresData[skill].scores[SCORE_NUM_PER_SKILL - 1].score = newScore;
	scoresData[skill].scores[SCORE_NUM_PER_SKILL - 1].text.setFillColor(sf::Color::Green);
	sortScoresArray(scoresData[skill].scores);
	createScoreTexts();
}

void ScoresGraphics::sortScoresArray(std::array<ScoreInfo, SCORE_NUM_PER_SKILL>& scores)
{
	std::sort(scores.begin(), scores.end(), compareScores);
}

void ScoresGraphics::outputScoresToFile(std::ofstream& errorLog, const PrefPathFinder& prefPath)
{
	writeSingleFile(errorLog, prefPath, EASY_FILE_NAME, 0);
	writeSingleFile(errorLog, prefPath, INTERM_FILE_NAME, 1);
	writeSingleFile(errorLog, prefPath, HARD_FILE_NAME, 2);
}

void ScoresGraphics::writeSingleFile(std::ofstream& errorLog, const PrefPathFinder& prefPath, const std::string& fileName, unsigned skill)
{
	if( std::ofstream bestScoreFile{prefPath.getPath() + fileName} )
	{
		for( unsigned i{0} ; i < SCORE_NUM_PER_SKILL - 1 ; ++i )
		{
			bestScoreFile << scoresData[skill].scores[i].score << " ";
		}
	}
	else{
		errorLog << "Error: couldn't open file '" << prefPath.getPath() << fileName << " to write best scores data.\n";
	}
}

bool compareScores(const ScoreInfo& first, const ScoreInfo& second)
{
	return first.score > second.score;
}